@@ -251,6 +251,17 @@ int io_bind(const struct sockaddr *addr, int type, const endpoint_flags_t *flags
 	return fd;
 }
 
+/// Optionally set a socket option and log error on failure.
+static void set_so(int fd, int so_option, int value, const char *descr)
+{
+	if (!value) return;
+	if (setsockopt(fd, SOL_SOCKET, so_option, &value, sizeof(value))) {
+		kr_log_error(IO, "failed to set %s to %d: %s\n",
+				descr, value, strerror(errno));
+		// we treat this as non-critical failure
+	}
+}
+
 int io_listen_udp(uv_loop_t *loop, uv_udp_t *handle, int fd)
 {
 	if (!handle) {
@@ -262,6 +273,10 @@ int io_listen_udp(uv_loop_t *loop, uv_udp_t *handle, int fd)
 	ret = uv_udp_open(handle, fd);
 	if (ret) return ret;
 
+	struct network *net = &the_worker->engine->net;
+	set_so(fd, SO_SNDBUF, net->listen_udp_buflens.snd, "UDP send buffer size");
+	set_so(fd, SO_RCVBUF, net->listen_udp_buflens.rcv, "UDP receive buffer size");
+
 	uv_handle_t *h = (uv_handle_t *)handle;
 	check_bufsize(h);
 	/* Handle is already created, just create context. */
@@ -702,6 +717,18 @@ int io_listen_tcp(uv_loop_t *loop, uv_tcp_t *handle, int fd, int tcp_backlog, bo
 	}
 #endif
 
+	/* These get inherited into the individual connections (on Linux at least). */
+	struct network *net = &the_worker->engine->net;
+	set_so(fd, SO_SNDBUF, net->listen_tcp_buflens.snd, "TCP send buffer size");
+	set_so(fd, SO_RCVBUF, net->listen_tcp_buflens.rcv, "TCP receive buffer size");
+#ifdef TCP_USER_TIMEOUT
+	val = net->tcp.user_timeout;
+	if (val && setsockopt(fd, IPPROTO_TCP, TCP_USER_TIMEOUT, &val, sizeof(val))) {
+		kr_log_error(IO, "listen TCP (user_timeout): %s\n", strerror(errno));
+	}
+	// TODO: also for upstream connections, at least this one option?
+#endif
+
 	handle->data = NULL;
 	return 0;
 }
@@ -964,11 +991,7 @@ static void xdp_rx(uv_poll_t* handle, int status, int events)
 	kr_require(xhd && xhd->session && xhd->socket);
 	uint32_t rcvd;
 	knot_xdp_msg_t msgs[XDP_RX_BATCH_SIZE];
-	int ret = knot_xdp_recv(xhd->socket, msgs, XDP_RX_BATCH_SIZE, &rcvd
-			#if KNOT_VERSION_HEX >= 0x030100
-			, NULL
-			#endif
-			);
+	int ret = knot_xdp_recv(xhd->socket, msgs, XDP_RX_BATCH_SIZE, &rcvd, NULL);
 
 	if (kr_fails_assert(ret == KNOT_EOK)) {
 		/* ATM other error codes can only be returned when called incorrectly */
@@ -1048,10 +1071,7 @@ int io_listen_xdp(uv_loop_t *loop, struct endpoint *ep, const char *ifname)
 
 	// This call is a libknot version hell, unfortunately.
 	int ret = knot_xdp_init(&xhd->socket, ifname, ep->nic_queue,
-		#if KNOT_VERSION_HEX < 0x030100
-			ep->port ? ep->port : KNOT_XDP_LISTEN_PORT_ALL,
-			KNOT_XDP_LOAD_BPF_MAYBE
-		#elif KNOT_VERSION_HEX < 0x030200
+		#if KNOT_VERSION_HEX < 0x030200
 			ep->port ? ep->port : (KNOT_XDP_LISTEN_PORT_PASS | 0),
 			KNOT_XDP_LOAD_BPF_MAYBE
 		#else
