@@ -1,5 +1,6 @@
 -- SPDX-License-Identifier: GPL-3.0-or-later
 -- Module interface
+local kres = require('kres')
 local ffi = require('ffi')
 local C = ffi.C
 local M = { layer = { } }
@@ -150,7 +151,8 @@ function M.layer.consume(state, req, pkt)
 				req.pool)
 		end
 	end
-	ffi.C.kr_ranked_rrarray_finalize(req.answ_selected, qry.uid, req.pool);
+	ffi.C.kr_ranked_rrarray_finalize(req.answ_selected, qry.uid, req.pool)
+	req:set_extended_error(kres.extended_error.FORGED, "BHD4: DNS64 synthesis")
 end
 
 local function hexchar2int(char)
@@ -175,7 +177,7 @@ function M.layer.produce(_, req, pkt)
 		then return end
 	-- Update packet question if it was minimized.
 	qry.flags.NO_MINIMIZE = true
-	if not ffi.C.knot_dname_is_equal(pkt.wire + 12, sname) then
+	if not ffi.C.knot_dname_is_equal(pkt.wire + 12, sname) or not pkt:has_wire() then
 		if not pkt:recycle() or not pkt:question(sname, qry.sclass, qry.stype)
 			then return end
 	end
